#!/usr/bin/python

import json
import mock
import os
import shutil
import subprocess
import unittest

os.environ['CHARM_DIR'] = os.path.join(os.path.dirname(__file__), '..')
os.environ['JUJU_UNIT_NAME'] = 'logstash-forwarder/0'

from charmhelpers.core import hookenv, services
from charmhelpers.core import host
import charmhelpers.fetch
subprocess.check_output = mock.MagicMock()
subprocess.check_call = mock.MagicMock()
subprocess.call = mock.MagicMock(return_value=0)
config = hookenv.Config({"application_name": "logstash_forwarder",
                         "config_dir": "/etc/logstash-forwarder",
                         "package_name": "logstash-forwarder_0.4.0_amd64.deb",
                         })
hookenv.config = lambda k: config.get(k)
shutil.copyfile = mock.MagicMock()
os.chmod = mock.MagicMock()

host.mkdir = mock.MagicMock()
host.service_start = mock.MagicMock()
host.service_stop = mock.MagicMock()
host.service_restart = mock.MagicMock()
host.write_file = mock.MagicMock()
hookenv.log = mock.MagicMock()

charmhelpers.fetch.apt_install = mock.MagicMock()
import hooks
hooks.conf.config = config
hooks.log = mock.MagicMock()


class TestInstall(unittest.TestCase):
    '''Testing that there are no exceptions in hooks.install'''

    def test_install_does_not_raise(self):
        with mock.patch('__builtin__.open'):
            hooks.install()

    def test_installs_packages(self):
        subprocess.call.reset()
        with mock.patch('__builtin__.open'):
            hooks.install()
        subprocess.call.assert_called_with(["dpkg", "-i",
                                            os.path.join(
                                                hookenv.charm_dir(),
                                                "files",
                                                hookenv.config('package_name'))
                                            ])

    def test_package_file_exist(self):
        pkgFile = os.path.join(hookenv.charm_dir(), "files",
                               hookenv.config('package_name'))
        if not os.path.exists(pkgFile):
            self.assert_fail()


class TestLogsRelation(unittest.TestCase):

    def patch(self, patch):
        m = patch.start()
        self.addCleanup(patch.stop)
        return m

    def setUp(self):
        self.patch(mock.patch.dict(os.environ, {
            'JUJU_HOOK_NAME': 'logs-relation-joined',
            'JUJU_MODEL_NAME': 'test-model',
        }))

        self.mhookenv = self.patch(
            mock.patch.object(services.helpers, 'hookenv')
        )
        self.mhookenv.relation_ids.return_value = ['baz']
        self.mhookenv.related_units.side_effect = lambda i: [i + '/0']
        self.mhookenv.relation_get.side_effect = [
            {'files': '\n'.join(['file1', 'file2']),
             'types': '\n'.join(['type1', 'type2'])}
        ]
        self.mock_gethostname = self.patch(mock.patch('socket.gethostname'))
        self.mock_gethostname.return_value = 'HOSTNAME'

        self.mrelations = self.patch(
            mock.patch.object(hookenv, 'relations_of_type')
        )
        self.relations = {'juju-info': [{'__unit__': 'remote/0'}]}
        self.mrelations.side_effect = self.relations.get

        host.write_file.reset_mock()
        host.service_restart.reset_mock()

    def test_logs_hooks_with_juju_info_relation(self):
        conf_path = hookenv.config("config_dir") + "/logstash_forwarder.conf"
        host.write_file.reset_mock()
        hooks.logs_relation_hooks()
        self.assertTrue(host.write_file.called)
        self.assertEqual(conf_path, host.write_file.call_args[0][0])
        conf = json.loads(host.write_file.call_args[0][1])
        self.assertEqual("file1", conf["files"][0]["paths"][0])
        expected_fields = {
            u'juju_service': u'remote',
            u'juju_unit': u'remote-0',
            u'juju_model': u'test-model',
            u'hostname': u'HOSTNAME',
            u'type': u'type1',
        }
        self.assertEqual(expected_fields, conf["files"][0]["fields"])
        host.service_restart.assert_called_once_with('logstash-forwarder')

    def test_logs_hooks_with_logs_relation(self):
        conf_path = hookenv.config("config_dir") + "/logstash_forwarder.conf"
        self.relations['logs'] = [{'__unit__': 'logs-remote/0'}]
        hooks.logs_relation_hooks()
        self.assertTrue(host.write_file.called)
        self.assertEqual(conf_path, host.write_file.call_args[0][0])
        conf = json.loads(host.write_file.call_args[0][1])
        self.assertEqual("file1", conf["files"][0]["paths"][0])
        expected_fields = {
            u'juju_service': u'logs-remote',
            u'juju_unit': u'logs-remote-0',
            u'juju_model': u'test-model',
            u'hostname': u'HOSTNAME',
            u'type': u'type1',
        }
        self.assertEqual(expected_fields, conf["files"][0]["fields"])
        host.service_restart.assert_called_once_with('logstash-forwarder')

    def test_logs_relation_simple(self):
        data = hooks.logs_relation()
        expected = {
            'file1': {'type': 'type1'},
            'file2': {'type': 'type2'},
        }
        self.assertEqual(expected, data)

    def test_logs_relation_extra(self):
        self.mhookenv.relation_get.side_effect = [
            {'files': 'file1 file2',
             'types': 'type1 type2',
             'service': 'svc1 svc2',
            }
        ]
        data = hooks.logs_relation()
        expected = {
            'file1': {'type': 'type1', 'service': 'svc1'},
            'file2': {'type': 'type2', 'service': 'svc2'},
        }
        self.assertEqual(expected, data)

    def test_logs_relation_extra_fewer(self):
        self.mhookenv.relation_get.side_effect = [
            {'files': 'file1 file2',
             'types': 'type1 type2',
             'service': 'svc1',
            }
        ]
        data = hooks.logs_relation()
        expected = {
            'file1': {'type': 'type1', 'service': 'svc1'},
            'file2': {'type': 'type2'},
        }
        self.assertEqual(expected, data)

    def test_logs_relation_extra_skip(self):
        self.mhookenv.relation_get.side_effect = [
            {'files': 'file1 file2',
             'types': 'type1 type2',
             'service': 'SKIP svc2',
            }
        ]
        data = hooks.logs_relation()
        expected = {
            'file1': {'type': 'type1'},
            'file2': {'type': 'type2', 'service': 'svc2'},
        }
        self.assertEqual(expected, data)

    def test_logs_relation_extra_no_private_address(self):
        self.mhookenv.relation_get.side_effect = [
            {'files': 'file1 file2',
             'types': 'type1 type2',
             'private-address': '1.2.3.4',
            }
        ]
        data = hooks.logs_relation()
        expected = {
            'file1': {'type': 'type1'},
            'file2': {'type': 'type2'},
        }
        self.assertEqual(expected, data)




